

tracks_map = {
    "Darlington Raceway": "intermediate",
    "World Wide Technology Raceway": "intermediate",
    "Homestead-Miami Speedway": "intermediate",
    "Bowman Gray Stadium": "short track",
    "Richmond Raceway": "short track",
    "Auto Club Speedway": "superspeedway",
    "Road America": "road course",
    "North Wilkesboro Speedway": "short track",
    "Las Vegas Motor Speedway": "intermediate",
    "Los Angeles Memorial Coliseum": "short track",
    "Sonoma Raceway": "road course",
    "Daytona International Speedway": "superspeedway",
    "Dover Motor Speedway": "intermediate",
    "Talladega Superspeedway": "superspeedway",
    "Rockingham Speedway": "intermediate",
    "Charlotte Motor Speedway": "intermediate",
    "Michigan International Speedway": "superspeedway",
    "Mid-Ohio Sports Car Course": "road course",
    "New Hampshire Motor Speedway": "short track",
    "DAYTONA Road Course": "road course",
    "Pocono Raceway": "superspeedway",
    "Knoxville Raceway": "short track",
    "Portland International Raceway": "road course",
    "Nashville Superspeedway": "intermediate",
    "Watkins Glen International": "road course",
    "Circuit of The Americas": "road course",
    "Iowa Speedway": "short track",
    "Dover International Speedway": "intermediate",
    "Lime Rock Park": "road course",
    "Bristol Motor Speedway": "short track",
    "Indianapolis Motor Speedway": "superspeedway",
    "Chicago Street Race": "road course",
    "Autódromo Hermanos Rodríguez": "road course",
    "Charlotte Motor Speedway Road Course": "road course",
    "Lucas Oil Indianapolis Raceway Park": "short track",
    "The Milwaukee Mile": "intermediate",
    "Indianapolis Motor Speedway Road Course": "road course",
    "Texas Motor Speedway": "intermediate",
    "Atlanta Motor Speedway": "intermediate",
    "Milwaukee Mile Speedway": "intermediate",
    "Kansas Speedway": "intermediate",
    "Bristol Motor Speedway Dirt": "short track",
    "Martinsville Speedway": "short track",
    "Phoenix Raceway": "short track"
}

tracks_by_type = {
    "superspeedway": [
        "Auto Club Speedway",
        "Daytona International Speedway",
        "Talladega Superspeedway",
        "Michigan International Speedway",
        "Pocono Raceway",
        "Indianapolis Motor Speedway"
    ],
    "intermediate": [
        "Darlington Raceway",
        "World Wide Technology Raceway",
        "Homestead-Miami Speedway",
        "Las Vegas Motor Speedway",
        "Rockingham Speedway",
        "Charlotte Motor Speedway",
        "Nashville Superspeedway",
        "Dover Motor Speedway",
        "Dover International Speedway",
        "The Milwaukee Mile",
        "Milwaukee Mile Speedway",
        "Texas Motor Speedway",
        "Atlanta Motor Speedway",
        "Kansas Speedway"
    ],
    "short track": [
        "Bowman Gray Stadium",
        "Richmond Raceway",
        "North Wilkesboro Speedway",
        "Los Angeles Memorial Coliseum",
        "New Hampshire Motor Speedway",
        "Knoxville Raceway",
        "Iowa Speedway",
        "Bristol Motor Speedway",
        "Lucas Oil Indianapolis Raceway Park",
        "Bristol Motor Speedway Dirt",
        "Martinsville Speedway",
        "Phoenix Raceway"
    ],
    "road course": [
        "Road America",
        "Sonoma Raceway",
        "Mid-Ohio Sports Car Course",
        "DAYTONA Road Course",
        "Portland International Raceway",
        "Watkins Glen International",
        "Circuit of The Americas",
        "Lime Rock Park",
        "Chicago Street Race",
        "Autódromo Hermanos Rodríguez",
        "Charlotte Motor Speedway Road Course",
        "Indianapolis Motor Speedway Road Course"
    ]
}